#ifndef UTILITIEC_TRACINGHEAP_H
#define UTILITIEC_TRACINGHEAP_H

#include "../allocator-interface/allocator-interface.h"

#include <stdalign.h>

enum TracingColor {
	TRACINGCOLOR_BLACK,
	TRACINGCOLOR_GREY,
	TRACINGCOLOR_WHITE,
};

typedef struct TracingObject {
	struct TracingObject* global_next;
	struct TracingObject* global_prev;
	struct TracingObject* color_next;
	struct TracingObject* color_prev;
	enum TracingColor color;

	// data member of the requested size
	alignas(void*) char data[];
} TracingObject;

typedef int    (*TracingHeapVisit) (void* context, void* reference);
typedef int    (*TracingHeapTraceData) (void* context, void* data, TracingHeapVisit trace_callback, void* callback_context);
typedef int    (*TracingHeapDataDestructor) (void* context, void* data);
typedef size_t (*TracingHeapSizeQuery) (void* context, void* data);

typedef struct TracingHeapConfig_s {
	TracingHeapTraceData trace_data;
	void* trace_context;

	TracingHeapDataDestructor destructor;
	void* destructor_context;

	TracingHeapSizeQuery size_query;
	void* size_query_context;
} TracingHeapConfig;

typedef struct TracingHeap_s {
	TracingObject* objects;
	TracingObject* white_objects;
	TracingObject* grey_objects;
	TracingObject* black_objects;

	enum TracingColor reachable_color;
	enum TracingColor unreachable_color;

	TracingHeapConfig config;
	allocator_t* allocator;
} TracingHeap;

int TracingHeap_Create(TracingHeap* self, allocator_t* allocator);
void TracingHeap_Destroy(TracingHeap* self);

void* TracingHeap_Allocate(TracingHeap* self, size_t bytes);

int    TracingHeap_BeginTrace(TracingHeap* self);
int    TracingHeap_EndTrace(TracingHeap* self);
int    TracingHeap_TraceNext(TracingHeap* self);
size_t TracingHeap_TraceNextN(TracingHeap* self, size_t n);
int    TracingHeap_AddTracingRoot(TracingHeap* self, void* data);

#endif // header
