#ifndef ARGUMENTC_H
#define ARGUMENTC_H

// There is a problem with short options, if you pop them out of order
// -ab 1 -s 2
// popshortarg s
// popshortarg a v
// popshortarg b ^ these race for the argument '1'

#include "../dynamicarray/dynamicarray.h"
#include "../StringView/StringView.h"

enum OptionType {
	OPTIONTYPE_NONE,
	OPTIONTYPE_SHORT,	// -s
	OPTIONTYPE_LONG,	// --long
	OPTIONTYPE_ARGUMENT,	// value
	OPTIONTYPE_DELIM,	// "--"
};

typedef struct Option_s {
	enum OptionType type;
	StringView content;
} Option;

typedef struct OptionArgument_s {
	Option option;
	Option argument;
} OptionArgument;

typedef struct Argumentc_s {
	DynamicArray array;
} Argumentc;

int Argumentc_Create(Argumentc* target, int argc, const char** argv);
void Argumentc_Destroy(Argumentc* argumentc);

bool Argumentc_HaveNextOption(Argumentc* argumentc);
Option Argumentc_PopNextOption(Argumentc* argumentc);

bool Argumentc_HaveOption(Argumentc* argumentc, StringView name);

Option Argumentc_PopShortOption(Argumentc* argumentc, StringView name);
Option Argumentc_PopLongOption(Argumentc* argumentc, StringView name);

OptionArgument Argumentc_PopShortArgument(Argumentc* argumentc, StringView name);
OptionArgument Argumentc_PopLongArgument(Argumentc* argumentc, StringView name);

bool Argumentc_PopLongOptionSwitch(Argumentc* argumentc, StringView name, bool bdefault);
bool Argumentc_PopShortOptionSwitch(Argumentc* argumentc, StringView name, bool bdefault);

int Argumentc_PopLongOptionInt(Argumentc* argumentc, StringView name, int idefault);
int Argumentc_PopShortOptionInt(Argumentc* argumentc, StringView name, int idefault);

const char* OptionType_ToString(enum OptionType type);

#endif
